var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
export var State;
(function (State) {
    State["OK"] = "OK";
    State["ERROR"] = "ERROR";
})(State || (State = {}));
export var Validity;
(function (Validity) {
    Validity["VALID"] = "VALID";
    Validity["INVALID"] = "INVALID";
    Validity["UNKNOWN"] = "UNKNOWN";
})(Validity || (Validity = {}));
export var Security;
(function (Security) {
    Security["ALLOWED"] = "ALLOWED";
    Security["FORBIDDEN"] = "FORBIDDEN";
    Security["UNKNOWN"] = "UNKNOWN";
})(Security || (Security = {}));
export var Persistence;
(function (Persistence) {
    Persistence["NEW"] = "NEW";
    Persistence["DIRTY"] = "DIRTY";
    Persistence["SAVED"] = "SAVED";
    Persistence["SAVE_ERROR"] = "SAVE_ERROR";
})(Persistence || (Persistence = {}));
/**
 * Transform an item to an object than can be saved.
 * undefined values (not be modify on ozone)
 * @param item
 */
export function toPatch(item) {
    let patch = {};
    for (let prop of Object.getOwnPropertyNames(item)) {
        if (prop in ['_meta', 'creationUser', 'modificationUser']) {
            continue;
        }
        const val = item[prop];
        if (val !== undefined) {
            patch[prop] = val;
        }
    }
    return patch;
}
/**
 * Transform an item to an object than can be saved
 * undefined value will be set to null (erase on ozone)
 * @param item
 */
export function toPatchWithUndefinedAsNull(item) {
    let patch = {};
    for (let prop of Object.getOwnPropertyNames(item)) {
        if (prop in ['_meta', 'creationUser', 'modificationUser']) {
            continue;
        }
        const val = item[prop];
        if (val === undefined) {
            patch[prop] = null;
        }
        else {
            patch[prop] = val;
        }
    }
    return patch;
}
let Item = class Item {
    constructor(src) {
        if (src) {
            this.id = src.id;
            this.version = src.version;
            this.type = src.type;
            this._meta = src._meta;
            this.name = src.name;
            this.deleted = src.deleted;
            this.traits = src.traits;
            this.tenant = src.tenant;
            this.creationUser = src.creationUser;
            this.modificationUser = src.modificationUser;
        }
    }
};
Item = __decorate([
    OzoneType('item'),
    __metadata("design:paramtypes", [Item])
], Item);
export { Item };
export class GenericItem extends Item {
}
/**
 * Decorator for class that extend ozone item
 * @param typeIdentifier
 * @constructor
 */
export function OzoneType(typeIdentifier) {
    return function (constructor) {
        return class extends constructor {
            constructor() {
                super(...arguments);
                this.type = typeIdentifier;
            }
        };
    };
}
